/*
 * File         : RestrictedConference.cpp
 * Author       : Pawel Roman (proman@npac.syr.edu)
 * Created      : Wed Feb 23 11:06:26 1998
 * Revision     : $Revision: 1.2 $
 * Updated      : $Date: 1998/03/17 22:01:18 $
 * Copyright: Northeast Parallel Architectures Center
 *            at Syracuse University 1998
 */

#include "RestrictedConference.h"
#include "Participant.h"
#include "ConferenceRep.h"
#include "ConferenceFactory.h"
#include "ActiveUser.h"
#include "DirectoryProtocol.h"

RestrictedConference::
~RestrictedConference()
{
	ACE_WRITE_GUARD( ACE_RW_Thread_Mutex, mon, lock_ );
	ACE_DEBUG(( LM_DEBUG, "(%t) deletng conference: 0x%x\n", this ));
	if( SDSConferenceRep::instance()->unbind( key() ) == -1 )
		ACE_ERROR(( LM_ERROR, "(%t) [!](RestrictedConference::~RestrictedConference) unbinding error: 0x%x\n", this ));
	else
		ACE_DEBUG(( LM_DEBUG, "(%t) Conference 0x%x unbound\n", this ));
}

int RestrictedConference::
set_chair( Participant* part )
{
	ACE_WRITE_GUARD_RETURN( ACE_RW_Thread_Mutex, mon, lock_, -1 );
	ACE_DEBUG(( LM_DEBUG, "(%t) setting the chair: 0x%x for conference: 0x%x\n", part, this ));
	participants_.push_back( part );
	part->conf( this );
	return 0;
}

int RestrictedConference::
join( Participant* part )
{
	ACE_WRITE_GUARD_RETURN( ACE_RW_Thread_Mutex, mon, lock_, -1 );
	ACE_DEBUG(( LM_DEBUG, "(%t) Participant: 0x%x tries to join conference: 0x%x\n", part, this ));
	DSParticipantRep::iterator it = find( participants_.begin(), participants_.end(), part );
	if( it != participants_.end() )
	{
		part->send_msg( DirectoryProtocol::INFO, "You have already joined this conference" );
		ACE_ERROR_RETURN(( LM_ERROR, "[!](RestrictedConference::join) Part 0x%x already joined conf: 0x%x\n", part, this ), -1 );
	}
	Conference* cur_conf = part->conf();
	if( cur_conf )
	{
		part->send_msg( DirectoryProtocol::INFO, "You should leave your current conference first" );
		ACE_ERROR_RETURN(( LM_ERROR, "[!](RestrictedConference::join) Part 0x%x should leave conf: 0x%x first\n", part, cur_conf ), -1 );
	}
	if( participants_.size() == 0 )
		return accept( part );
	else
	{
		ConferenceKey keys[2] = { key(), part->key() };
		participants_[0]->send_msg( DirectoryProtocol::ACCEPT, &keys, sizeof keys );
//		part->send_msg( DirectoryProtocol::INFO, "Wait for being accepted" );
	}
	return 0;
}

int RestrictedConference::
accept( Participant* part )
{
	ACE_WRITE_GUARD_RETURN( ACE_RW_Thread_Mutex, mon, lock_, -1 );
	ACE_DEBUG(( LM_DEBUG, "(%t) Participant: 0x%x joining conference: 0x%x\n", part, this ));
	Conference* cur_conf = part->conf();
	if( cur_conf )
	{
		part->send_msg( DirectoryProtocol::INFO, "You should leave your current conference first" );
		ACE_ERROR_RETURN(( LM_ERROR, "[!](RestrictedConference::join) Part 0x%x should leave conf: 0x%x first\n", part, cur_conf ), -1 );
	}
	part->conf( this );
	participants_.push_back( part );
	ConferenceKey keys[2] = { key(), part->key() };
	part->bcast_msg( DirectoryProtocol::JOIN, &keys, sizeof keys );
	return 0;
}

int RestrictedConference::
leave( Participant* part )
{
	ACE_WRITE_GUARD_RETURN( ACE_RW_Thread_Mutex, mon, lock_, -1 );
	ACE_DEBUG(( LM_DEBUG, "(%t) Participant: 0x%x left conference: 0x%x\n", part, this ));
	DSParticipantRep::iterator it = find( participants_.begin(), participants_.end(), part );
	if( it == participants_.end() )
		ACE_ERROR_RETURN(( LM_ERROR, "[!](RestrictedConference::leave) cannot find 0x%x in conf: 0x%x\n", part, this ), -1 );

	part->conf( 0 );
	ConferenceKey keys[2] = { key(), part->key() };
	part->bcast_msg( DirectoryProtocol::LEAVE, &keys, sizeof keys );
	if( participants_[0] == part )
	{
		for( size_t idx = 1; idx < participants_.size(); idx++ )
		{
			Participant* other_participant = participants_[idx];
			other_participant->conf( 0 );
			keys[1] = other_participant->key();
			other_participant->send_msg( DirectoryProtocol::LEAVE, &keys, sizeof keys );
		}
		part->bcast_msg( DirectoryProtocol::DELCONF, &keys[0], sizeof keys[0] );
		closed_ = true;
	}
	participants_.erase( it );
	return 0;
}

int RestrictedConference::
build_key_vector( KeyVector keys )
{
	ACE_READ_GUARD_RETURN( ACE_RW_Thread_Mutex, mon, lock_, -1 );
	DSParticipantRep::size_type num_part = participants_.size();
	size_t topic_size = strlen( topic_.c_str() ) + 1;
	size_t keys_size = sizeof( ConferenceFactory::ConferenceType )
						+ sizeof( ConferenceKey )
						+ topic_size
						+ sizeof( ParticipantKey ) * num_part;

	ACE_NEW_RETURN( keys, char[keys_size], 0 );
	//	*( ConferenceFactory::ConferenceType* )keys = ConferenceFactory::OPEN;
	ConferenceFactory::ConferenceType conf_type = ConferenceFactory::OPEN;
	memcpy( keys, &conf_type, sizeof conf_type );

	//	*( ConferenceKey* )( keys + sizeof( ConferenceFactory::ConferenceType ) ) = key();
	ConferenceKey conf_key = key();
	memcpy( keys + sizeof( ConferenceFactory::ConferenceType ), &conf_key, sizeof conf_key );

	memcpy( keys + sizeof( ConferenceFactory::ConferenceType ) + sizeof( ConferenceKey ), topic_.c_str(), topic_size );
	ParticipantKey* part_keys = ( ParticipantKey* )( keys + sizeof( ConferenceFactory::ConferenceType )
														  + sizeof( ConferenceKey )
														  + topic_size );

	for( DSParticipantRep::size_type idx = 0; idx != num_part; idx++ )
	{
		ACE_DEBUG(( LM_DEBUG, "(%t) building key vector, part: 0x%x for conf: 0x%x\n",
					participants_[idx], this ));

		//		part_keys[idx] = participants_[idx]->key();
		ParticipantKey pkey = participants_[idx]->key();
		memcpy( part_keys, &pkey, sizeof pkey );
		part_keys += sizeof pkey;
	}
	return keys_size;
}
