/*
 * File         : msgPASSWD.h
 * Author       : Pawel Roman (proman@npac.syr.edu)
 * Created      : Wed Feb 20 18:06:26 1998
 * Revision     : $Revision: 1.1 $
 * Updated      : $Date: 1998/03/13 00:01:22 $
 * Copyright: Northeast Parallel Architectures Center
 *            at Syracuse University 1998
 */

#if !defined(__MSGPASSWD_H)
#define __MSGPASSWD_H

#include "DirectoryProtocol.h"
#include "Message.h"
#include "ProtectedConference.h"

template<class HEADER>
class msgPASSWD
: public Message<HEADER>
{
public:
	msgPASSWD();
	~msgPASSWD();
	int recv( SAP& sap, HEADER& header );
	int send( SAP& sap );
	int set( void* params, size_t len = 0 );
	int execute();
	Message<HEADER>* clone() { return new msgPASSWD<HEADER>; }

private:
	char* buf_;
};

template<class HEADER>
inline msgPASSWD<HEADER>::
msgPASSWD()
: Message<HEADER>( HEADER( DirectoryProtocol::PASSWD ) ), buf_( 0 )
{
}

template<class HEADER>
msgPASSWD<HEADER>::
~msgPASSWD() 
{
	delete[] buf_; 
}

template<class HEADER>
int msgPASSWD<HEADER>::
recv( SAP& sap, HEADER& header )
{
	delete[] buf_;
	if( !( buf_ = new char[header.msg_length()] ) )
		ACE_ERROR_RETURN(( LM_ERROR, "[!](msgPASSWD<HEADER>::recv) allocation error\n" ), -1 );

	if( sap.recv_n( buf_, header.msg_length() ) != header.msg_length() )
		ACE_ERROR_RETURN(( LM_ERROR, "[!](msgPASSWD<HEADER>::recv) recv error\n" ), -1 );

	header_ = header;
	return 0;
}

template<class HEADER>
int msgPASSWD<HEADER>::
send( SAP& sap )
{
	if( header_.send( sap ) == -1 )
		ACE_ERROR_RETURN(( LM_ERROR, "[!](msgPASSWD<HEADER>::send) header send error\n" ), -1 );

	if( sap.send_n( buf_, header_.msg_length() ) != header_.msg_length() )
		ACE_ERROR_RETURN(( LM_ERROR, "[!](msgPASSWD<HEADER>::send) body send error\n" ), -1 );

	return 0;
}

template<class HEADER>
int msgPASSWD<HEADER>::
set( void* conf_key, size_t len )
{
	const size_t key_size = sizeof( ConferenceKey );
	if( len != key_size )
		ACE_ERROR_RETURN(( LM_ERROR, "[!](msgPASSWD<HEADER>::set) setting error\n" ), -1 );

	delete[] buf_;
	ACE_NEW_RETURN( buf_, char[key_size], -1 );
	memcpy( buf_, conf_key, key_size );
	header_.msg_length( key_size );
	return 0;
}

template<class HEADER>
int msgPASSWD<HEADER>::
execute()
{
	if( header_.msg_length() <= sizeof( ConferenceKey ) )
		ACE_ERROR_RETURN(( LM_ERROR, "[!](msgPASSWD<HEADER>::execute) message body too short\n" ), -1 );

	ConferenceKey conf_key = *( ConferenceKey* )buf_;
	const char* passwd = buf_ + sizeof conf_key;
	Conference* conf;
	DSConferenceIterator it( *SDSConferenceRep::instance() );
	it.acquire_read();
	if( SDSConferenceRep::instance()->find( conf_key, conf ) == -1 )
	{
		it.release();
		ACE_ERROR_RETURN(( LM_ERROR, "[!](msgPASSWD<HEADER>::execute) conf %d not found\n", conf_key ), -1 );
	}
	ACE_DEBUG(( LM_DEBUG, "(%t) executing PASSWD msg 0x%x, user: 0x%x, conf: 0x%x\n",
				this, user_, conf ));

	( ( ProtectedConference* )conf )->join( user_, passwd );
	it.release();
	return 0;
}

#endif
